@@ -26,7 +26,11 @@
 #include <libosso-abook/osso-abook.h>
 #include <libringtoned/ringtoned.h>
 #include "contact.h"
+#include "i18n.h"
 #include "ringtone-chooser.h"
+#include "ringtone.h"
+#include "settings-dialog.h"
+#include "special-ringtones-settings.h"
 
 typedef struct
 {
@@ -89,7 +93,7 @@ ringtoned_saved_cb (OssoABookContact *contact,
                 contact ? osso_abook_contact_get_uid (contact) : "<unset>",
                 status);
         hildon_banner_show_information (NULL, NULL,
-                "Failed to save the custom ringtone");
+                _("Impossible to save the custom ringtone"));
     }
 
     old_contact = osso_abook_menu_extension_get_contact (closure->ext);
@@ -104,18 +108,6 @@ ringtoned_saved_cb (OssoABookContact *contact,
         GtkWindow *parent = osso_abook_menu_extension_get_parent (closure->ext);
         gtk_widget_destroy (GTK_WIDGET (parent));
     }
-    else if (closure->button)
-    {
-        const OssoABookMenuEntry *entries;
-
-        /* Update the button text, it's "set custom ringtone" if there is
-         * no ringtone currently set, but "change custom rigtone" if a
-         * ringtone is already set */
-        entries = osso_abook_menu_extension_get_menu_entries (closure->ext);
-        g_assert (entries);
-        g_assert (entries[0].label);
-        gtk_button_set_label (GTK_BUTTON (closure->button), entries[0].label);
-    }
 
     set_ringtone_closure_free (closure);
 }
@@ -136,6 +128,24 @@ dialog_response_cb (GtkDialog          *dialog,
         pcr_contact_set_custom_ringtone (contact, ringtone,
                 ringtoned_saved_cb, closure);
 
+        /* Update the button text, it's "set custom ringtone" if there is
+         * no ringtone currently set, but "change custom rigtone" if a
+         * ringtone is already set.
+         * We don't wait for the contact to be committed before doing it
+         * or the UI change would be visible if the user is quick enough
+         * and reopens the menu. */
+        if (closure->button)
+        {
+            const OssoABookMenuEntry *entries;
+
+            entries = osso_abook_menu_extension_get_menu_entries (closure->ext);
+            g_assert (entries);
+            g_assert (entries[0].label);
+
+            gtk_button_set_label (GTK_BUTTON (closure->button),
+                    entries[0].label);
+        }
+
         g_free (ringtone);
     }
     else
@@ -176,13 +186,17 @@ set_ringtone_cb (GtkWidget              *button,
 }
 
 static OssoABookMenuEntry set_ringtone_entries[] = {
-    { "Set custom ringtone", 0, 0,
-      G_CALLBACK (set_ringtone_cb), NULL },
+    { NULL, 0, 0, G_CALLBACK (set_ringtone_cb), NULL },
 };
+static const gchar *set_ringtone_entries_label = N_("Set custom ringtone");
 
 static OssoABookMenuEntry change_ringtone_entries[] = {
-    { "Change custom ringtone", 0, 0,
-      G_CALLBACK (set_ringtone_cb), NULL },
+    { NULL, 0, 0, G_CALLBACK (set_ringtone_cb), NULL },
+};
+static const gchar *change_ringtone_entries_label = N_("Change custom ringtone");
+
+static OssoABookMenuEntry empty_entries[] = {
+    { NULL, 0, 0, NULL, NULL },
 };
 
 static const OssoABookMenuEntry *
@@ -190,35 +204,64 @@ get_menu_entries (OssoABookMenuExtension *ext)
 {
     OssoABookContact *contact;
     const gchar *ringtone = PCR_RINGTONE_DEFAULT;
+    OssoABookMenuEntry *ret;
+    const gchar *label;
 
     contact = osso_abook_menu_extension_get_contact (ext);
-    if (contact)
-        ringtone = pcr_contact_get_custom_ringtone (contact);
+    if (!contact || OSSO_ABOOK_IS_VOICEMAIL_CONTACT (contact))
+        return empty_entries;
 
-    if (g_strcmp0 (ringtone, PCR_RINGTONE_DEFAULT) == 0)
-        return set_ringtone_entries;
+    contact = osso_abook_menu_extension_get_contact (ext);
+    ringtone = pcr_contact_get_custom_ringtone (contact);
+
+    if (pcr_ringtone_is_default (ringtone))
+    {
+        ret =  set_ringtone_entries;
+        label = set_ringtone_entries_label;
+    }
     else
-        return change_ringtone_entries;
+    {
+        ret = change_ringtone_entries;
+        label = change_ringtone_entries_label;
+    }
+
+    if (!ret[0].label)
+        ret[0].label = _(label);
+
+    return ret;
 }
 
 static int
 get_n_menu_entries (OssoABookMenuExtension *ext)
 {
+    OssoABookContact *contact;
+
+    contact = osso_abook_menu_extension_get_contact (ext);
+    if (!contact || OSSO_ABOOK_IS_VOICEMAIL_CONTACT (contact))
+        return 0;
+
     g_assert (G_N_ELEMENTS (set_ringtone_entries) ==
               G_N_ELEMENTS (change_ringtone_entries));
-
     return G_N_ELEMENTS (set_ringtone_entries);
 }
 
 static void
 per_contact_ringtone_extension_class_init (PerUserRingtoneExtensionClass *cls)
 {
+    static gboolean hook_added = FALSE;
     OssoABookMenuExtensionClass *ext_class;
 
     ext_class = OSSO_ABOOK_MENU_EXTENSION_CLASS (cls);
     ext_class->get_n_menu_entries = get_n_menu_entries;
     ext_class->get_menu_entries = get_menu_entries;
-    ext_class->name =  OSSO_ABOOK_MENU_NAME_CONTACT_VIEW;
+    ext_class->name = OSSO_ABOOK_MENU_NAME_CONTACT_VIEW;
+
+    if (!hook_added)
+    {
+        pcr_settings_dialog_add_widget_hook (
+                special_ringtones_settings_widget_hook);
+        hook_added = TRUE;
+    }
 }
 
 static void
